//////////////////////////////////////////////////////////////////////////////
// Program : zwhpt.cc
// Author  : Graeme Watt
// Purpose : Calculate transverse momentum distribution of W,Z,H bosons in
//           p-p(bar) collisions using doubly-unintegrated parton distributions.
//           See hep-ph/0309096.
// Started : 12/08/03
// Modified: 13/09/04 First public version.
//////////////////////////////////////////////////////////////////////////////

#include "TFile.h"
#include "TH1.h"
#include "zwhpt.h"

// Function prototypes:
void getMuSq(double x1,double x2,double beta1,double beta2,
	     double &mu1sq,double &mu2sq);
double sumPDFs(double u1,double d1,double s1,double c1,double b1,double ubar1,double dbar1,
	       double u2,double d2,double s2,double c2,double b2,double ubar2,double dbar2);

// 1-D histograms:
TH1D *hTot,*hK;

///////////////////
// Main program: //
///////////////////

int main(int argc, char *argv[])
{
  double DoIntegration(gsl_monte_function *vegas_integrand,
		       double *xl,double *xu,const size_t dim,
		       size_t iniCalls);
  double integrand(double *v,size_t dim,void *params);
  sIntegrandParams IP,*pIP=&IP;
  pIP->iniCalls=10000;
  
  const int nargs=4; // number of arguments expected on command line
  char *outputfile;  // name of ROOT file to write to
  
#ifdef ZPROD
  if (argc!=nargs) {
    cout << "ZProd.  Wrong number of arguments.  Usage is:" << endl;
    cout << " zprod rootfile PTRunMin PTRunMax" << endl;
    return(0);
  }
  else
    cout << "Welcome to ZProd!" << endl;
#elif WPROD
  if (argc!=nargs) {
    cout << "WProd.  Wrong number of arguments.  Usage is:" << endl;
    cout << " wprod rootfile PTRunMin PTRunMax" << endl;
    return(0);
  }
  else
    cout << "Welcome to WProd!" << endl;
#elif HPROD
  if (argc!=nargs) {
    cout << "HProd.  Wrong number of arguments.  Usage is:" << endl;
    cout << " hprod rootfile PTRunMin PTRunMax" << endl;
    return(0);
  }
  else
    cout << "Welcome to HProd!" << endl;
#endif
  
  outputfile = argv[1];
  double PTRunMin = atof(argv[2]), PTRunMax = atof(argv[3]);
  
#ifdef ZPROD
//   const size_t nBins = 1;
//   const double PTMin = 0., PTMax = 1000.;
  const size_t nBins = 300;
  const double PTMin = 0., PTMax = 300.;
#elif WPROD
//   const size_t nBins = 1;
//   const double PTMin = 0., PTMax = 1000.;
  const size_t nBins = 200;
  const double PTMin = 0., PTMax = 200.;
#elif HPROD
//   const size_t nBins = 1;
//   const double PTMin = 0., PTMax = 1000.;
  const size_t nBins = 200;
  const double PTMin = 0., PTMax = 200.;
#endif

  TFile *rootfile = new TFile(outputfile,"RECREATE");
  hTot = new TH1D("hTot","Total",nBins,PTMin,PTMax);
  hK = new TH1D("hK","K-factor",nBins,PTMin,PTMax);

  const double xMin = 0.0, xMax = 1.0, zMin = 0.0, zMax = 1.0;
  const double ktSqMin = 0.0, ktSqMax = 1.e6;
  const double phiMin = 0.0, phiMax = 2.*M_PI;
  double total,KFactor;
  
  pIP->ktSqMax = ktSqMax;

  for (size_t bin=1; bin<=nBins; bin++) {

    // Get PT limits of current bin:
    double PTBinMin = hK->GetBinLowEdge(bin);
    double PTBinMax = hK->GetBinLowEdge(bin) + hK->GetBinWidth(bin);
    double PTBinWidth = hK->GetBinWidth(bin);
    double PTBinCenter = hK->GetBinCenter(bin);
    if (PTBinMin<PTRunMin || PTBinMax>PTRunMax) continue;
    cout << "PT = " << PTBinMin << " to " << PTBinMax << " GeV" << endl;

    // Total contribution:
    size_t dim = 6;
    gsl_monte_function vegas_integrand = { &integrand, dim, pIP };
    double xl[] = { xMin, ktSqMin, phiMin, PTBinMin, zMin, zMin };
    double xu[] = { xMax, ktSqMax, phiMax, PTBinMax, zMax, zMax };
    total = DoIntegration(&vegas_integrand, xl, xu, dim, pIP->iniCalls)/PTBinWidth;
    cout << "dsigma/dPT = " << total << endl;
    hTot->Fill(PTBinCenter,total);

    // Use scale in K-factor from Kulesza-Stirling paper:
#ifdef ZPROD
    KFactor = exp(0.5*pdf1.alpha_S(pow(MZ,2./3.)*pow(PTBinCenter,4./3.))*4./3.*M_PI);
#elif WPROD
    KFactor = exp(0.5*pdf1.alpha_S(pow(MW,2./3.)*pow(PTBinCenter,4./3.))*4./3.*M_PI);
#elif HPROD
    KFactor = exp(0.5*pdf1.alpha_S(pow(MH,2./3.)*pow(PTBinCenter,4./3.))*3.*M_PI);
#endif
    hK->Fill(PTBinCenter,KFactor);
    cout << "K-factor = " << KFactor << endl << endl;

  }

  rootfile->Write();
  rootfile->Close();

  return(0);
}

//////////////////////////
// End of main program. //
//////////////////////////


////////////////
// Integrand: //
////////////////

double integrand(double *v, size_t dim, void *params)
{
  static const sIntegrandParams* pIP = (sIntegrandParams *)params;
  if (!pIP) error("integrand","pIP",pIP);

  if (dim!=6) error("integrand","dim",dim);

  const double x1=v[0], kt1sq=v[1], phi=v[2], qt=v[3];

  double z1=0.0, z2=0.0;
  z1=v[4]; z2=v[5];

  if (x1<0.||x1>1.) error("integrand","x1",x1);
  if (z1<0.||z1>1.) error("integrand","z1",z1);
  if (z2<0.||z2>1.) error("integrand","z2",z2);
  
  if (z1<x1) return 0.;

  const double kt1 = sqrt(kt1sq), qtsq = qt*qt;
  const double r1 = kt1sq/S, R = qtsq/S, beta1 = r1*z1/x1/(1.-z1);
  if (beta1<0.||beta1>1.) return 0.;

  double totdcross=0.0;

  const double kt2sqrtarg = qtsq - kt1sq*pow(sin(phi),2);
  if (kt2sqrtarg<0.) return 0.;
  
  // Loop over two solutions for kt2
  for (size_t kt2pm=0; kt2pm<=1; kt2pm++) {

    double kt2;
    if (kt2pm==0) kt2 = -kt1*cos(phi) + sqrt(kt2sqrtarg);
    else kt2 = -kt1*cos(phi) - sqrt(kt2sqrtarg);
    double kt2sq = kt2*kt2, r2 = kt2sq/S;

    if (kt2<0.0 || kt2>sqrt(pIP->ktSqMax)) continue;

    double tempvar = R + TAU + x1*beta1 + z2*r2/(1.-z2);
    double x2sqrtarg = pow(tempvar,2)-4.*x1*beta1*z2*r2/(1.-z2);
    if (x2sqrtarg<0.) return 0.;

    // Loop over two solutions for x2
    for (size_t x2pm=0; x2pm<=1; x2pm++) {

      // Impose constraints 0<beta2<x1<z1<1 and 0<beta1<x2<z2<1:
      double x2;
      if (x2pm==0) x2 = (tempvar + sqrt(x2sqrtarg))/(2.*x1);
      else x2 = (tempvar - sqrt(x2sqrtarg))/(2.*x1);
      if (x2<0.||x2>1.) continue;
      if (z2<x2) continue;
      
      double beta2 = r2*z2/x2/(1.-z2);
      if (beta2<0.||beta2>1.) continue;
      if (x1<beta2||x2<beta1) continue;
    
      // Get mu1sq and mu2sq
      double mu1sq,mu2sq;
      getMuSq(x1,x2,beta1,beta2,mu1sq,mu2sq);
    
#ifdef HPROD

      double g1 = pdf1.calc(7,x1,z1,kt1sq,mu1sq);
            
      double g2 = pdf2.calc(7,x2,z2,kt2sq,mu2sq);
      
      double dcross = g1*g2;  // on-shell matrix element
      dcross *= x1*x2*S/MHSQ; // corrected on-shell matrix element!

#else

      double u1,d1,s1,c1,b1,ubar1,dbar1;
      u1 = pdf1.calc(0,x1,z1,kt1sq,mu1sq);
      d1 = pdf1.calc(1,x1,z1,kt1sq,mu1sq);
      s1 = pdf1.calc(2,x1,z1,kt1sq,mu1sq);
      c1 = pdf1.calc(3,x1,z1,kt1sq,mu1sq);
      b1 = pdf1.calc(4,x1,z1,kt1sq,mu1sq);
      ubar1 = pdf1.calc(5,x1,z1,kt1sq,mu1sq);
      dbar1 = pdf1.calc(6,x1,z1,kt1sq,mu1sq);
      
      double u2,d2,s2,c2,b2,ubar2,dbar2;
      u2 = pdf2.calc(0,x2,z2,kt2sq,mu2sq);
      d2 = pdf2.calc(1,x2,z2,kt2sq,mu2sq);
      s2 = pdf2.calc(2,x2,z2,kt2sq,mu2sq);
      c2 = pdf2.calc(3,x2,z2,kt2sq,mu2sq);
      b2 = pdf2.calc(4,x2,z2,kt2sq,mu2sq);
      ubar2 = pdf2.calc(5,x2,z2,kt2sq,mu2sq);
      dbar2 = pdf2.calc(6,x2,z2,kt2sq,mu2sq);
      
      double dcross = sumPDFs(u1,d1,s1,c1,b1,ubar1,dbar1,
			      u2,d2,s2,c2,b2,ubar2,dbar2);
	
#endif
      
      if (x2pm==1) warning("integrand","x2pm",x2pm);

      dcross *= 1./kt2/kt1sq;
      dcross *= 2.*qt/fabs(kt2+kt1*cos(phi));
      dcross /= (x1*x2-beta1*beta2);
      totdcross += dcross;
    
    } // End loop over x2pm

  } // End loop over kt2pm    
    
  totdcross *= prefactor/(2.*M_PI*x1);
  
  return totdcross;
}


// Get mu1sq and mu2sq:
void getMuSq(double x1,double x2,double beta1,double beta2,
	     double &mu1sq,double &mu2sq)
{
  const double Xi = (x2-beta1)/(x1-beta2);
  
  mu1sq = x1*x1*S*Xi;
  mu2sq = x2*x2*S/Xi;
  
  return;
}


#ifndef HPROD
// Get weighted sum of PDFs:
double sumPDFs(double u1,double d1,double s1,double c1,double b1,double ubar1,double dbar1,
	       double u2,double d2,double s2,double c2,double b2,double ubar2,double dbar2)
{
  double sbar1 = s1, cbar1 = c1, bbar1 = b1, sbar2 = s2, cbar2 = c2, bbar2 = b2;

#ifdef ZPROD
  const double uptype = ( VUSQ + AQSQ )*( u1*u2 + c1*c2 + ubar1*ubar2 + cbar1*cbar2 );
  const double downtype = ( VDSQ + AQSQ )*( d1*d2 + s1*s2 + b1*b2 + dbar1*dbar2 + sbar1*sbar2 + bbar1*bbar2 );
  double dcross = uptype+downtype;
#elif WPROD
  double dcross = VUDSQ * ( u1*d2 + d1*u2 + ubar1*dbar2 + dbar1*ubar2 )
    + VUSSQ * ( u1*s2 + s1*u2 + ubar1*sbar2 + sbar1*ubar2 )
    + VUBSQ * ( u1*b2 + b1*u2 + ubar1*bbar2 + bbar1*ubar2 )
    + VCDSQ * ( c1*d2 + d1*c2 + cbar1*dbar2 + dbar1*cbar2 )
    + VCSSQ * ( c1*s2 + s1*c2 + cbar1*sbar2 + sbar1*cbar2 )
    + VCBSQ * ( c1*b2 + b1*c2 + cbar1*bbar2 + bbar1*cbar2 );
#endif
  
  return dcross;
}
#endif


////////////////////////////////////////////////////////////////////////////
// Integrate using VEGAS until integral converges with required accuracy: //
////////////////////////////////////////////////////////////////////////////

double DoIntegration(gsl_monte_function *vegas_integrand,double *xl,double *xu,
		     const size_t dim,size_t iniCalls)
{
  // VEGAS variables
  const gsl_rng_type *rng_type; gsl_rng *rng;
  gsl_monte_vegas_state *vegas_state = gsl_monte_vegas_alloc (dim);
  gsl_rng_env_setup ();
  rng_type = gsl_rng_default;
  rng = gsl_rng_alloc (rng_type);

  size_t its=0, calls=iniCalls; // minimum number of function calls
  double res,err,relerr,chisq;
  
  if (iniCalls<1000) error("DoIntegration","iniCalls (<1000)",iniCalls);

  do {
    
    // VEGAS warm-up:
    vegas_state->verbose = -1;
    vegas_state->stage = 0;
    gsl_monte_vegas_integrate (vegas_integrand, xl, xu, dim, calls/5,
			       rng, vegas_state, &res, &err);
    // Now use warmed-up grid:
    vegas_state->stage = 1;

    do {
      gsl_monte_vegas_integrate (vegas_integrand, xl, xu, dim, calls,
				 rng, vegas_state, &res, &err);
      its += (vegas_state->iterations);
      vegas_state->stage = 2;

      // If integral identically zero (dangerous if too few initial steps!):
      if (res==0.) {
	chisq = 1.0; relerr = 0.0;
	warning("DoIntegration","res",res);
      }
      else { chisq = vegas_state->chisq; relerr = fabs(err/res); }
      
    } while (((fabs(chisq - 1.0) > MAXVEGCHISQ) || (relerr > MAXVEGRELERR))
	     && (its < MAXVEGITS));

    cout << "   calls = " << calls << ", sigma = " << res
	 << ", relerr = " << relerr << ", chisq = " << chisq << endl;

    its = 0; calls *= 2; // increase number of calls by 2

  } while (((fabs(chisq - 1.0) > MAXVEGCHISQ) || (relerr > MAXVEGRELERR))
	   && (calls<MAXVEGCALLS));
  
  if ((fabs(chisq - 1.0) > MAXVEGCHISQ) || (relerr > MAXVEGRELERR)) {
    warning("DoIntegration","calls",calls);
    cerr << "   calls = " << calls << ", sigma = " << res
    	 << ", relerr = " << relerr << ", chisq = " << chisq << endl;
  }

  gsl_rng_free (rng);      
  gsl_monte_vegas_free (vegas_state);

  return res;
}
