// Header file for ZWHPT
// Comments to: watt(at)hep.ucl.ac.uk
// This code calculates the PT distributions of W and Z bosons 
// at the Tevatron and Higgs bosons at the LHC: hep-ph/0309096.

#ifndef _ZPROD_H_INCLUDED_

#include <parton.h>
#include <gsl/gsl_monte.h>
#include <gsl/gsl_monte_vegas.h>

// GLOBAL VARIABLES:
// Warning: also global variables in header files.

// global unintegrated parton objects:
// static class cUnintParton pdf1,pdf2; // Don't interpolate Sudakov form factors.

// Or, to speed up code, interpolate Sudakov form factors from a grid.
// (But takes time to initialise.)
static class cUnintParton pdf1(true),pdf2(true);

static const sPdfConsts consts = pdf1.getConsts();
static const sPdfContent* intpdf1 = pdf1.getContent();
static const sPdfContent* intpdf2 = pdf2.getContent();

// Physical constants:
static const double HBARC2=0.389379292e9; // (hbar*c)^2 in units of GeV^2 pbarn [PDG]
static const double GF=1.16639E-5;        // Fermi coupling constant in GeV^-2 [PDG]
static const double NCOL = 3.;            // number of colours
#ifdef ZPROD
static const double MZ = 91.1876;         // Z boson mass [PDG]
static const double MZSQ = MZ*MZ;         // Z boson mass squared
static const double ROOTS=1800.;          // Tevatron Run I centre-of-mass energy
static const double S=ROOTS*ROOTS;        //
static const double TAU = MZSQ/S;         //
static const double BRZTOEE = 0.03363;    // B.R.(Z -> e^+ e^-) [PDG]
static const double prefactor = TAU*GF*sqrt(2.)*M_PI*HBARC2*BRZTOEE/NCOL;
static const double SIN2THETAW=0.23113;   // Weak-mixing angle [PDG]
static const double VUSQ = pow(1./2.-4./3.*SIN2THETAW,2); // vector coupling to
static const double VDSQ = pow(-1./2.+2./3.*SIN2THETAW,2);//  u and d quarks
static const double AQSQ = 1./4.;         // axial vector coupling squared
#elif WPROD
static const double MW = 80.423;          // W boson mass [PDG]
static const double MWSQ = MW*MW;         // W boson mass squared
static const double ROOTS=1800.;          // Tevatron Run I centre-of-mass energy
static const double S=ROOTS*ROOTS;        //
static const double TAU = MWSQ/S;         //
static const double BRWTOENU = 0.1072;    // B.R.(W -> e \nu) [PDG]
static const double prefactor = TAU*GF*sqrt(2.)*M_PI*HBARC2*BRWTOENU/NCOL;
// CKM matrix elements squared from PDG:
static const double VUDSQ = pow(0.9734,2);  // |V_{ud}|^2
static const double VUSSQ = pow(0.2196,2);  // |V_{us}|^2
static const double VUBSQ = pow(3.6e-3,2);  // |V_{ub}|^2
static const double VCDSQ = pow(0.224,2);   // |V_{cd}|^2
static const double VCSSQ = pow(0.996,2);   // |V_{cs}|^2
static const double VCBSQ = pow(41.2e-3,2); // |V_{cb}|^2
#elif HPROD
static const double MH = 125.;            // Higgs boson mass (arbitrary)
static const double MHSQ = MH*MH;         // Higgs boson mass squared
static const double ROOTS=14000.;         // LHC centre-of-mass energy
static const double S=ROOTS*ROOTS;        //
static const double TAU = MHSQ/S;         //
static const double prefactor = TAU*GF*sqrt(2.)*HBARC2*pow(pdf1.alpha_S(MHSQ),2)/576./M_PI;
#endif

// VEGAS constants:
static const size_t MAXVEGITS=15;       // Max. number of VEGAS iterations
static const double MAXVEGRELERR=0.001; // Max. VEGAS relative error
static const double MAXVEGCHISQ=0.5;    // Max. discrepancy of VEGAS chisq from 1
static const size_t MAXVEGCALLS=size_t(1e10); // Max. number of VEGAS calls

// STRUCTURE DEFINITIONS:
struct sIntegrandParams {
  size_t iniCalls;
  double ktSqMax;
};

#define _ZPROD_H_INCLUDED_
#endif
